import {
  Controller,
  Get,
  Post,
  Body,
  Patch,
  Param,
  Delete,
  NotFoundException,
  HttpCode,
  BadRequestException,
} from '@nestjs/common';
import { ApiResponse } from '@nestjs/swagger';
import { DepartmentService } from './department.service';
import { Department } from './entities/department.entity';
import { DepartmentCreateDto } from './dto/department-create.dto';
import { DepartmentUpdateDto } from './dto/department-update.dto';
import { EntityNotFoundError } from 'typeorm';

@Controller()
export class DepartmentController {
  constructor(private readonly departmentService: DepartmentService) {}

  private handleEntityNotFoundError(id: number, e: Error) {
    if (e instanceof EntityNotFoundError) {
      throw new NotFoundException(`Not found: id=${id}`);
    } else {
      throw e;
    }
  }

  @Get()
  @ApiResponse({ type: [Department] })
  findAll(): Promise<Department[]> {
    return this.departmentService.findAll();
  }

  @Get(':id')
  @ApiResponse({ type: Department })
  async findOne(@Param('id') id: number): Promise<Department | null> {
    // NOTE: the + operator returns the numeric representation of the object.
    return this.departmentService.findOne(+id).catch((err) => {
      this.handleEntityNotFoundError(id, err);
      return null;
    });
  }

  @Post()
  @ApiResponse({ type: [Department] })
  create(@Body() departmentCreateDto: DepartmentCreateDto) {
    return this.departmentService.create(departmentCreateDto);
  }

  @Patch(':id')
  // Status 204 because no content will be returned
  @HttpCode(204)
  async update(
    @Param('id') id: number,
    @Body() departmentUpdateDto: DepartmentUpdateDto,
  ): Promise<void> {
    if (Object.keys(departmentUpdateDto).length === 0) {
      throw new BadRequestException('Request body is empty');
    }
    await this.departmentService
      .update(+id, departmentUpdateDto)
      .catch((err) => {
        this.handleEntityNotFoundError(id, err);
      });
  }

  @Delete(':id')
  @HttpCode(204)
  async remove(@Param('id') id: number): Promise<void> {
    await this.departmentService.remove(+id).catch((err) => {
      this.handleEntityNotFoundError(id, err);
    });
  }
}
